# -*- coding: utf-8 -*-

# Copyright (C) 2006 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""TMPlayer file."""

import aeidon
import re

__all__ = ("TMPlayer",)


class TMPlayer(aeidon.SubtitleFile):

    """TMPlayer file."""

    format = aeidon.formats.TMPLAYER
    mode = aeidon.modes.TIME
    _re_one_digit_hour = re.compile(r"^-?\d:\d\d:\d\d:")
    _re_two_digit_hour = re.compile(r"^-?\d\d:\d\d:\d\d:")

    def __init__(self, path, encoding, newline=None):
        """Initialize an :class:`TMPlayer` instance."""
        aeidon.SubtitleFile.__init__(self, path, encoding, newline)
        self.two_digit_hour = True

    def copy_from(self, other):
        """Copy generic properties from `other` file."""
        aeidon.SubtitleFile.copy_from(self, other)
        if self.format != other.format: return
        self.two_digit_hour = other.two_digit_hour

    def read(self):
        """
        Read file and return subtitles.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        """
        subtitles = [self._get_subtitle()]
        for line in self._read_lines():
            match = self._re_one_digit_hour.search(line)
            if match is not None:
                i = match.span()[1]
                subtitle = self._get_subtitle()
                sign = "-" if line.startswith("-") else ""
                time = line[:i-1] + ".000"
                if time.startswith("-"):
                    time = time[1:]
                time = sign + "0" + time
                subtitle.start_time = time
                subtitles[-1].end_time = time
                subtitle.main_text = line[i:].replace("|", "\n")
                subtitles.append(subtitle)
                self.two_digit_hour = False
            match = self._re_two_digit_hour.search(line)
            if match is not None:
                i = match.span()[1]
                subtitle = self._get_subtitle()
                subtitle.start_time = line[:i-1] + ".000"
                subtitles[-1].end_time = subtitle.start_time
                subtitle.main_text = line[i:].replace("|", "\n")
                subtitles.append(subtitle)
                self.two_digit_hour = True
        subtitles[-1].duration_seconds = 5
        return subtitles[1:]

    def write_to_file(self, subtitles, doc, f):
        """
        Write `subtitles` from `doc` to file `f`.

        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        for subtitle in subtitles:
            start = subtitle.calc.round(subtitle.start_time, 0)
            start = (start[:-4] if self.two_digit_hour
                     else ("-" + start[2:-4]
                           if start.startswith("-")
                           else start[1:-4]))

            text = subtitle.get_text(doc).replace("\n", "|")
            f.write("{}:{}\n".format(start, text))



# -*- coding: utf-8 -*-

# Copyright (C) 2011 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Allow use of non-standard framerates."""

import aeidon
import gaupol
import os

from aeidon.i18n   import _
from gi.repository import Gtk


class AddFramerateDialog(gaupol.BuilderDialog):

    """Dialog for entering a custom framerate value."""

    _widgets = ["spin_button"]

    def __init__(self, parent):
        """Initialize an :class:`AddFramerateDialog` instance."""
        directory = os.path.abspath(os.path.dirname(__file__))
        ui_file_path = os.path.join(directory, "add-framerate-dialog.ui")
        gaupol.BuilderDialog.__init__(self, ui_file_path)
        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
        self.add_button(_("_Add"), Gtk.ResponseType.OK)
        self.set_default_response(Gtk.ResponseType.OK)
        self.set_transient_for(parent)
        self.set_modal(True)

    def _on_response(self, *args):
        """Update spin button before dispatching response."""
        self._spin_button.update()

    def get_framerate(self):
        """Return framerate entered in the spin button."""
        return float(self._spin_button.get_value())


class PreferencesDialog(gaupol.BuilderDialog):

    """Dialog for editing a list of custom framerates."""

    _widgets = ["add_button", "remove_button", "toolbar", "tree_view"]

    def __init__(self, framerates, parent):
        """Initialize a :class:`PreferencesDialog` instance."""
        directory = os.path.abspath(os.path.dirname(__file__))
        ui_file_path = os.path.join(directory, "preferences-dialog.ui")
        gaupol.BuilderDialog.__init__(self, ui_file_path)
        self.set_default_response(Gtk.ResponseType.CLOSE)
        self.set_transient_for(parent)
        self.set_modal(True)
        self._init_toolbar()
        self._init_tree_view(framerates)
        self._remove_button.set_sensitive(False)
        gaupol.util.scale_to_content(self._tree_view,
                                     min_nchar=30,
                                     max_nchar=60,
                                     min_nlines=8,
                                     max_nlines=16)

    def get_framerates(self):
        """Return the defined custom framerates."""
        framerates = []
        store = self._tree_view.get_model()
        for i in range(len(store)):
            framerates.append(store[i][0])
        return sorted(framerates)

    def _get_selected_rows(self):
        """Return a sequence of the selected rows."""
        selection = self._tree_view.get_selection()
        paths = selection.get_selected_rows()[1]
        return list(map(gaupol.util.tree_path_to_row, paths))

    def _init_toolbar(self):
        """Initialize the tree view inline toolbar."""
        self._toolbar.set_icon_size(Gtk.IconSize.MENU)
        style = self._toolbar.get_style_context()
        style.add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)
        theme = Gtk.IconTheme.get_default()
        # Tool buttons in the UI file are specified as symbolic icons
        # by name, found in adwaita-icon-theme, if missing in another
        # theme fall back to non-symbolic icons.
        if not all((theme.has_icon(self._add_button.get_icon_name()),
                    theme.has_icon(self._remove_button.get_icon_name()))):
            self._add_button.set_icon_name("list-add")
            self._remove_button.set_icon_name("list-remove")

    def _init_tree_view(self, framerates):
        """Initialize the tree view."""
        selection = self._tree_view.get_selection()
        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        selection.connect("changed", self._on_tree_view_selection_changed)
        store = Gtk.ListStore(float)
        for framerate in framerates:
            store.append((framerate,))
        store.set_sort_column_id(0, Gtk.SortType.ASCENDING)
        self._tree_view.set_model(store)
        renderer = Gtk.CellRendererText()
        renderer.props.xalign = 1
        column = Gtk.TreeViewColumn("", renderer, text=0)
        column.set_sort_column_id(0)
        def format_framerate(column, renderer, store, itr, data):
            renderer.props.text = "{:.6f}".format(store.get_value(itr, 0))
        column.set_cell_data_func(renderer, format_framerate)
        self._tree_view.append_column(column)

    def _on_add_button_clicked(self, *args):
        """Add a new framerate."""
        dialog = AddFramerateDialog(self._dialog)
        response = gaupol.util.run_dialog(dialog)
        framerate = dialog.get_framerate()
        dialog.destroy()
        if response != Gtk.ResponseType.OK: return
        store = self._tree_view.get_model()
        store.append((framerate,))

    def _on_remove_button_clicked(self, *args):
        """Remove the selected framerate."""
        rows = self._get_selected_rows()
        store = self._tree_view.get_model()
        for row in reversed(sorted(rows)):
            path = gaupol.util.tree_row_to_path(row)
            store.remove(store.get_iter(path))
        if len(store) <= 0: return
        self._tree_view.set_cursor(max(row-1, 0))

    def _on_tree_view_selection_changed(self, *args):
        """Set the remove button sensitivity."""
        selection = self._tree_view.get_selection()
        n = selection.count_selected_rows()
        self._remove_button.set_sensitive(n > 0)


class CustomFrameratesExtension(gaupol.Extension):

    """Allow use of non-standard framerates."""

    def __init__(self):
        """Initialize a :class:`CustomFrameratesExtension` instance."""
        self.application = None
        self.conf = None
        self.framerates = []

    def _add_framerates(self):
        """Add custom framerates and corresponding UI elements."""
        self.framerates = []
        menu = self.application.get_menubar_section(
            "custom-framerates-placeholder")
        for value in sorted(self.conf.framerates):
            name = "FPS_{:.3f}".format(value).replace(".", "_")
            if hasattr(aeidon.framerates, name): continue
            setattr(aeidon.framerates, name, aeidon.EnumerationItem())
            framerate = getattr(aeidon.framerates, name)
            framerate.label = _("{:.3f} fps").format(value)
            framerate.value = float(value)
            self.framerates.append(framerate)
            with aeidon.util.silent(AttributeError):
                # Menubar not available when running unit tests.
                action = "win.set-framerate::{}".format(name)
                menu.append(framerate.label, action)

    def _remove_framerates(self):
        """Remove custom framerates and corresponding UI elements."""
        fallback = aeidon.framerates.FPS_23_976
        if gaupol.conf.editor.framerate in self.framerates:
            gaupol.conf.editor.framerate = fallback
        # Go through all application's pages and reset those set
        # to a custom framerate back to the default framerate.
        orig_page = self.application.get_current_page()
        for page in self.application.pages:
            if not page.project.framerate in self.framerates: continue
            self.application.set_current_page(page)
            action = self.application.get_action("set-framerate")
            action.activate(str(fallback))
        self.application.set_current_page(orig_page)
        for framerate in self.framerates:
            delattr(aeidon.framerates, str(framerate))
        with aeidon.util.silent(AttributeError):
            # Menubar not available when running unit tests.
            self.application.get_menubar_section(
                "custom-framerates-placeholder").remove_all()

    def setup(self, application):
        """Setup extension for use with `application`."""
        options = dict(framerates=[48.0])
        gaupol.conf.register_extension("custom_framerates", options)
        self.conf = gaupol.conf.extensions.custom_framerates
        self.application = application
        self.framerates = []
        self._add_framerates()

    def show_preferences_dialog(self, parent):
        """Show a dialog to edit list of custom framerates."""
        dialog = PreferencesDialog(self.conf.framerates, parent)
        gaupol.util.run_dialog(dialog)
        self.conf.framerates = list(dialog.get_framerates())
        dialog.destroy()
        self._remove_framerates()
        self._add_framerates()

    def teardown(self, application):
        """End use of extension with `application`."""
        self._remove_framerates()
        self.application = None
        self.conf = None
        self.framerates = []



# -*- coding: utf-8 -*-

# Copyright (C) 2006 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Observable versions of built-in mutable objects."""

import copy
import functools

__all__ = ("ObservableDict", "ObservableList", "ObservableSet",)


def _mutation(function):
    """Decorator for sending a notification after mutating object."""
    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        value = function(*args, **kwargs)
        args[0].master.notify(args[0].name)
        return value
    return wrapper


class ObservableDict(dict):

    """
    Observable version of ``dict``.

    :ivar master: Master instance with a ``notify`` method
    :ivar name: Argument passed when calling :attr:`master`'s ``notify`` method
    """

    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args[:-2], **kwargs)
        self.master = args[-2]
        self.name = args[-1]

    def __copy__(self):
        dic = dict(copy.copy(x) for  x in self.items())
        return self.__class__(dic, self.master, self.name)

    def __deepcopy__(self, memo):
        dic = dict(copy.deepcopy(x) for  x in self.items())
        return self.__class__(dic, self.master, self.name)

    @_mutation
    def __delitem__(self, *args, **kwargs):
        return dict.__delitem__(self, *args, **kwargs)

    @_mutation
    def __setitem__(self, *args, **kwargs):
        return dict.__setitem__(self, *args, **kwargs)

    @_mutation
    def clear(self, *args, **kwargs):
        return dict.clear(self, *args, **kwargs)

    @_mutation
    def pop(self, *args, **kwargs):
        return dict.pop(self, *args, **kwargs)

    @_mutation
    def popitem(self, *args, **kwargs):
        return dict.popitem(self, *args, **kwargs)

    @_mutation
    def setdefault(self, *args, **kwargs):
        return dict.setdefault(self, *args, **kwargs)

    @_mutation
    def update(self, *args, **kwargs):
        return dict.update(self, *args, **kwargs)


class ObservableList(list):

    """
    Observable version of ``list``.

    :ivar master: Master instance with a ``notify`` method
    :ivar name: Argument passed when calling :attr:`master`'s ``notify`` method
    """

    def __init__(self, *args, **kwargs):
        list.__init__(self, *args[:-2], **kwargs)
        self.master = args[-2]
        self.name = args[-1]

    def __copy__(self):
        lst = list(copy.copy(x) for x in self)
        return self.__class__(lst, self.master, self.name)

    def __deepcopy__(self, memo):
        lst = [copy.deepcopy(x) for x in self]
        return self.__class__(lst, self.master, self.name)

    @_mutation
    def __delitem__(self, *args, **kwargs):
        return list.__delitem__(self, *args, **kwargs)

    @_mutation
    def __iadd__(self, *args, **kwargs):
        return list.__iadd__(self, *args, **kwargs)

    @_mutation
    def __imul__(self, *args, **kwargs):
        return list.__imul__(self, *args, **kwargs)

    @_mutation
    def __setitem__(self, *args, **kwargs):
        return list.__setitem__(self, *args, **kwargs)

    @_mutation
    def append(self, *args, **kwargs):
        return list.append(self, *args, **kwargs)

    @_mutation
    def extend(self, *args, **kwargs):
        return list.extend(self, *args, **kwargs)

    @_mutation
    def insert(self, *args, **kwargs):
        return list.insert(self, *args, **kwargs)

    @_mutation
    def pop(self, *args, **kwargs):
        return list.pop(self, *args, **kwargs)

    @_mutation
    def remove(self, *args, **kwargs):
        return list.remove(self, *args, **kwargs)

    @_mutation
    def reverse(self, *args, **kwargs):
        return list.reverse(self, *args, **kwargs)

    @_mutation
    def sort(self, *args, **kwargs):
        return list.sort(self, *args, **kwargs)


class ObservableSet(set):

    """
    Observable version of ``set``.

    :ivar master: Master instance with a ``notify`` method
    :ivar name: Argument passed when calling :attr:`master`'s ``notify`` method
    """

    def __init__(self, *args, **kwargs):
        set.__init__(self, *args[:-2], **kwargs)
        self.master = args[-2]
        self.name = args[-1]

    def __copy__(self):
        zet = set(copy.copy(x) for x in self)
        return self.__class__(zet, self.master, self.name)

    def __deepcopy__(self, memo):
        zet = set(copy.deepcopy(x) for x in self)
        return self.__class__(zet, self.master, self.name)

    @_mutation
    def __iand__(self, *args, **kwargs):
        return set.__iand__(self, *args, **kwargs)

    @_mutation
    def __ior__(self, *args, **kwargs):
        return set.__ior__(self, *args, **kwargs)

    @_mutation
    def __isub__(self, *args, **kwargs):
        return set.__isub__(self, *args, **kwargs)

    @_mutation
    def __ixor__(self, *args, **kwargs):
        return set.__ixor__(self, *args, **kwargs)

    @_mutation
    def add(self, *args, **kwargs):
        return set.add(self, *args, **kwargs)

    @_mutation
    def clear(self, *args, **kwargs):
        return set.clear(self, *args, **kwargs)

    @_mutation
    def difference_update(self, *args, **kwargs):
        return set.difference_update(self, *args, **kwargs)

    @_mutation
    def discard(self, *args, **kwargs):
        return set.discard(self, *args, **kwargs)

    @_mutation
    def intersection_update(self, *args, **kwargs):
        return set.intersection_update(self, *args, **kwargs)

    @_mutation
    def pop(self, *args, **kwargs):
        return set.pop(self, *args, **kwargs)

    @_mutation
    def remove(self, *args, **kwargs):
        return set.remove(self, *args, **kwargs)

    @_mutation
    def symmetric_difference_update(self, *args, **kwargs):
        return set.symmetric_difference_update(self, *args, **kwargs)

    @_mutation
    def update(self, *args, **kwargs):
        return set.update(self, *args, **kwargs)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Cell renderer for time data in format ``[-]HH:MM:SS.SSS``."""

import gaupol

from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import Gtk

__all__ = ("TimeCellRenderer",)


class TimeCellRenderer(Gtk.CellRendererText):

    """Cell renderer for time data in format ``[-]HH:MM:SS.SSS``."""

    __gtype_name__ = "TimeCellRenderer"

    def __init__(self):
        """Initialize a :class:`TimeCellRenderer` instance."""
        GObject.GObject.__init__(self)
        self._in_editor_menu = False

    def do_start_editing(self, event, widget, path, bg_area, cell_area, flags):
        """Initialize and return a :class:`gaupol.TimeEntry` widget."""
        editor = gaupol.TimeEntry()
        gaupol.style.use_font(editor, "custom")
        editor.set_has_frame(False)
        editor.set_alignment(self.props.xalign)
        editor.set_text(self.props.text)
        editor.select_region(0, -1)
        editor.gaupol_path = path
        editor.connect("focus-out-event", self._on_editor_focus_out_event)
        editor.connect("key-press-event", self._on_editor_key_press_event)
        editor.connect("populate-popup",  self._on_editor_populate_popup)
        editor.show()
        return editor

    def _on_editor_focus_out_event(self, editor, *args):
        """End editing."""
        if self._in_editor_menu: return
        editor.remove_widget()
        self.emit("editing-canceled")

    def _on_editor_key_press_event(self, editor, event):
        """End editing if ``Enter`` or ``Escape`` pressed."""
        if (event.get_state() &
            (Gdk.ModifierType.SHIFT_MASK |
             Gdk.ModifierType.CONTROL_MASK)): return
        if event.keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter):
            editor.remove_widget()
            self.emit("edited", editor.gaupol_path, editor.get_text())
            return True
        if event.keyval == Gdk.KEY_Escape:
            editor.remove_widget()
            self.emit("editing-canceled")

    def _on_editor_populate_popup(self, editor, menu):
        """Disable "focus-out-event" ending editing."""
        self._in_editor_menu = True
        def on_menu_unmap(menu, self):
            self._in_editor_menu = False
        menu.connect("unmap", on_menu_unmap, self)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Writing subtitle data to file."""

import aeidon


class SaveAgent(aeidon.Delegate):

    """Writing subtitle data to file."""

    def _save(self, doc, file, keep_changes):
        """
        Write subtitle data from `doc` to `file`.

        Return indices of texts changed due to markup conversion.
        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        current_format = self.get_format(doc)
        orig_texts = [x.get_text(doc) for x in self.subtitles]
        indices = []
        if current_format is not None and file.format != current_format:
            # Convert markup if saving in different format.
            converter = aeidon.MarkupConverter(current_format, file.format)
            for i, subtitle in enumerate(self.subtitles):
                text = subtitle.get_text(doc)
                new_text = converter.convert(text)
                if new_text == text: continue
                subtitle.set_text(doc, new_text)
                indices.append(i)
        file.write(self.subtitles, doc)
        if keep_changes: return indices
        for i, subtitle in enumerate(self.subtitles):
            subtitle.set_text(doc, orig_texts[i])
        return []

    @aeidon.deco.export
    def save(self, doc, file=None, keep_changes=True):
        """
        Write subtitle data from `doc` to `file`.

        `file` can be ``None`` to use existing file.
        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        if doc == aeidon.documents.MAIN:
            return self.save_main(file, keep_changes)
        if doc == aeidon.documents.TRAN:
            return self.save_translation(file, keep_changes)
        raise ValueError("Invalid document: {!r}".format(doc))

    @aeidon.deco.export
    def save_main(self, file=None, keep_changes=True):
        """
        Write subtitle data from main document to `file`.

        `file` can be ``None`` to use :attr:`main_file`.
        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        file = file or self.main_file
        if file is not None and self.main_file is not None:
            file.copy_from(self.main_file)
        indices = self._save(aeidon.documents.MAIN, file, keep_changes)
        if keep_changes:
            if (self.main_file is not None and
                file.mode != self.main_file.mode):
                # Apply possibly changed mode (times vs. frames).
                for i, subtitle in enumerate(self.subtitles):
                    subtitle.mode = file.mode
                self.emit("positions-changed", self.get_all_indices())
            self.main_file = file
            self.main_changed = 0
            self.emit("main-texts-changed", indices)
        self.emit("main-file-saved", file)

    @aeidon.deco.export
    def save_translation(self, file=None, keep_changes=True):
        """
        Write subtitle data from translation document to `file`.

        `file` can be ``None`` to use :attr:`tran_file`.
        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        file = file or self.tran_file
        if file is not None and self.tran_file is not None:
            file.copy_from(self.tran_file)
        indices = self._save(aeidon.documents.TRAN, file, keep_changes)
        if keep_changes:
            self.tran_file = file
            self.tran_changed = 0
            self.emit("translation-texts-changed", indices)
        self.emit("translation-file-saved", file)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for checking and correcting spelling."""

import aeidon
import gaupol

from aeidon.i18n   import _
from gi.repository import Gtk
from gi.repository import Pango

__all__ = ("SpellCheckDialog",)


class SpellCheckDialog(gaupol.BuilderDialog):

    """
    Dialog for checking and correcting spelling.

    :ivar _doc: :attr:`aeidon.documents` item currectly being checked
    :ivar _entry_handler: Handler for replacement entry's "changed" signal
    :ivar _language: Language code for :attr:`_navigator`
    :ivar _navigator: :class:`aeidon.SpellCheckNavigator` instance used
    :ivar _new_rows: List of rows in the current page with changed texts
    :ivar _new_texts: List of changed texts in the current page
    :ivar _page: :class:`gaupol.Page` instance currectly being checked
    :ivar _pager: Iterator to iterate over all target pages
    :ivar _row: Row currently being checked
    :ivar _rower: Iterator to iterate over all rows
    """

    _widgets = [
        "add_button",
        "edit_button",
        "entry",
        "grid",
        "ignore_all_button",
        "ignore_button",
        "join_back_button",
        "join_forward_button",
        "replace_all_button",
        "replace_button",
        "text_view",
        "tree_view",
    ]

    def __init__(self, parent, application):
        """Initialize a :class:`SpellCheckDialog` instance."""
        gaupol.BuilderDialog.__init__(self, "spell-check-dialog.ui")
        self.application = application
        self._doc = None
        self._entry_handler = None
        self._language = gaupol.conf.spell_check.language
        self._language_name = aeidon.locales.code_to_name(self._language)
        self._navigator = aeidon.SpellCheckNavigator(self._language)
        self._new_rows = []
        self._new_texts = []
        self._page = None
        self._pager = None
        self._row = None
        self._rower = None
        self._init_dialog(parent)
        self._init_widgets()
        self._init_sensitivities()
        self.resize(*gaupol.conf.spell_check.size)
        self._pager = self._yield_pages()
        next(self._pager)
        self._proceed()

    def _append_changes(self):
        """Append current changes to the list of changes."""
        subtitle = self._page.project.subtitles[self._row]
        if self._navigator.text == subtitle.get_text(self._doc): return
        self._new_rows.append(self._row)
        self._new_texts.append(self._navigator.text)

    def _init_dialog(self, parent):
        """Initialize the dialog."""
        self.set_default_response(Gtk.ResponseType.CLOSE)
        self.set_transient_for(parent)
        self.set_modal(True)

    def _init_sensitivities(self):
        """Initialize widget sensitivities."""
        self._join_back_button.set_sensitive(False)
        self._join_forward_button.set_sensitive(False)
        self._replace_all_button.set_sensitive(False)
        self._replace_button.set_sensitive(False)

    def _init_tree_view(self):
        """Initialize the suggestion tree view."""
        selection = self._tree_view.get_selection()
        selection.set_mode(Gtk.SelectionMode.SINGLE)
        selection.connect("changed", self._on_tree_view_selection_changed)
        store = Gtk.ListStore(str)
        self._tree_view.set_model(store)
        column = Gtk.TreeViewColumn("", Gtk.CellRendererText(), text=0)
        self._tree_view.append_column(column)

    def _init_widgets(self):
        """Initialize widget properties."""
        self._init_tree_view()
        gaupol.style.use_font(self._entry, "custom")
        gaupol.style.use_font(self._text_view, "custom")
        gaupol.style.use_font(self._tree_view, "custom")
        with aeidon.util.silent(AttributeError):
            # Top and bottom margins available since GTK 3.18.
            self._text_view.set_top_margin(6)
            self._text_view.set_bottom_margin(6)
        text_buffer = self._text_view.get_buffer()
        text_buffer.create_tag("misspelled", weight=Pango.Weight.BOLD)
        scale = gaupol.util.scale_to_size
        scale(self._text_view, nchar=55, nlines=4, font="custom")
        scale(self._tree_view, nchar=20, nlines=6, font="custom")
        self._entry_handler = self._entry.connect("changed", self._on_entry_changed)

    def _on_add_button_clicked(self, *args):
        """Add the current word to personal word list."""
        self._navigator.add()
        self._proceed()

    def _on_edit_button_clicked(self, *args):
        """Edit the current text in a separate dialog."""
        text = self._navigator.text
        dialog = gaupol.TextEditDialog(self._dialog, text)
        response = gaupol.util.run_dialog(dialog)
        text = dialog.get_text()
        dialog.destroy()
        if response != Gtk.ResponseType.OK: return
        self._navigator.reset(text)
        self._proceed()

    def _on_entry_changed(self, entry):
        """Populate suggestions based on text in `entry`."""
        word = entry.get_text()
        suggestions = self._navigator.checker.suggest(word) if word else []
        self._populate_tree_view(suggestions, select=False)
        self._replace_button.set_sensitive(bool(word))
        self._replace_all_button.set_sensitive(bool(word))

    def _on_ignore_all_button_clicked(self, *args):
        """Ignore the current word and any subsequent instances."""
        self._navigator.ignore_all()
        self._proceed()

    def _on_ignore_button_clicked(self, *args):
        """Ignore the current word."""
        self._navigator.ignore()
        self._proceed()

    def _on_join_back_button_clicked(self, *args):
        """Join the current word with the previous."""
        self._navigator.join_with_previous()
        self._proceed()

    def _on_join_forward_button_clicked(self, *args):
        """Join the current word with the next."""
        self._navigator.join_with_next()
        self._proceed()

    def _on_replace_all_button_clicked(self, *args):
        """Replace the current word and subsequent instances with `replacement`."""
        self._navigator.replace_all(self._entry.get_text())
        self._proceed()

    def _on_replace_button_clicked(self, *args):
        """Replace the current word with `replacement`."""
        self._navigator.replace(self._entry.get_text())
        self._proceed()

    def _on_response(self, dialog, response):
        """Apply changes to the current page."""
        self._register_changes()
        self._save_geometry()
        self._set_done()

    def _on_tree_view_selection_changed(self, *args):
        """Copy the selected suggestion into the entry."""
        selection = self._tree_view.get_selection()
        store, itr = selection.get_selected()
        if itr is None: return
        path = store.get_path(itr)
        row = gaupol.util.tree_path_to_row(path)
        self._set_entry_text(store[row][0])

    def _populate_tree_view(self, suggestions, select=True):
        """Populate the tree view with `suggestions`."""
        store = self._tree_view.get_model()
        self._tree_view.set_model(None)
        store.clear()
        for suggestion in suggestions:
            store.append((suggestion,))
        self._tree_view.set_model(store)
        if select and len(store) > 0:
            self._tree_view.set_cursor(0)
            self._tree_view.scroll_to_cell(0)

    def _proceed(self):
        """Move on to the next spelling error."""
        try:
            # Move on to the next spelling error in the current text.
            return self._proceed_current()
        except StopIteration:
            # Commit changes to the current text.
            self._append_changes()
        try:
            # Move on to the next row.
            next(self._rower)
            return self._proceed()
        except StopIteration:
            # Commit changes to the current page.
            self._register_changes()
        try:
            # Move on to the next page.
            next(self._pager)
            # Use idle_add here to break out of recursion to
            # reduce the likelihood of hitting the recursion limit.
            return gaupol.util.idle_add(self._proceed)
        except StopIteration:
            self._set_done()

    def _proceed_current(self):
        """Move on to the next spelling error in the current text."""
        next(self._navigator)
        col = self._page.document_to_text_column(self._doc)
        self._page.view.set_focus(self._row, col)
        self._page.view.scroll_to_row(self._row)
        text_buffer = self._text_view.get_buffer()
        text_buffer.set_text(self._navigator.text)
        start = text_buffer.get_iter_at_offset(self._navigator.pos)
        end = text_buffer.get_iter_at_offset(self._navigator.endpos)
        text_buffer.apply_tag_by_name("misspelled", start, end)
        mark = text_buffer.create_mark(None, end, True)
        self._text_view.scroll_to_mark(mark=mark,
                                       within_margin=0,
                                       use_align=False,
                                       xalign=0.5,
                                       yalign=0.5)

        leading = self._navigator.leading_context(1)
        trailing = self._navigator.trailing_context(1)
        self._join_back_button.set_sensitive(leading.isspace())
        self._join_forward_button.set_sensitive(trailing.isspace())
        self._set_entry_text("")
        self._populate_tree_view(self._navigator.suggest())
        self._tree_view.grab_focus()

    def _register_changes(self):
        """Register changes to the current page."""
        if not self._new_rows: return
        self._page.project.replace_texts(
            self._new_rows, self._doc, self._new_texts)
        self._page.project.set_action_description(
            aeidon.registers.DO, _("Spell-checking"))
        self._new_rows = []
        self._new_texts = []

    def _save_geometry(self):
        """Save dialog size."""
        if self.is_maximized(): return
        gaupol.conf.spell_check.size = list(self.get_size())

    def _set_done(self):
        """Set state of widgets for finished spell-check."""
        self._text_view.get_buffer().set_text("")
        self._set_entry_text("")
        self._populate_tree_view([])
        self._grid.set_sensitive(False)
        self._navigator.checker.write_replacements()

    def _set_entry_text(self, word):
        """Set `word` to the entry with its "changed" handler blocked."""
        self._entry.handler_block(self._entry_handler)
        self._entry.set_text(word)
        self._entry.handler_unblock(self._entry_handler)
        self._replace_button.set_sensitive(bool(word))
        self._replace_all_button.set_sensitive(bool(word))

    def _yield_pages(self):
        """Yield pages to check spelling in."""
        field = gaupol.conf.spell_check.field
        doc = gaupol.util.text_field_to_document(field)
        target = gaupol.conf.spell_check.target
        for page in self.application.get_target_pages(target):
            if not page.project.subtitles: continue
            self.application.set_current_page(page)
            self._page = page
            self._doc = doc
            self._rower = self._yield_rows(page)
            next(self._rower)
            yield page

    def _yield_rows(self, page):
        """Yield rows to check spelling in."""
        for row, subtitle in enumerate(page.project.subtitles):
            self._row = row
            text = subtitle.get_text(self._doc)
            self._navigator.reset(text)
            yield row



# -*- coding: utf-8 -*-

# Copyright (C) 2006 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Data editing extension delegates of :class:`aeidon.Project`."""

from .clipboard import ClipboardAgent # noqa
from .edit      import EditAgent # noqa
from .format    import FormatAgent # noqa
from .open      import OpenAgent # noqa
from .position  import PositionAgent # noqa
from .preview   import PreviewAgent # noqa
from .register  import RegisterAgent # noqa
from .save      import SaveAgent # noqa
from .search    import SearchAgent # noqa
from .set       import SetAgent # noqa
from .text      import TextAgent # noqa
from .util      import UtilityAgent # noqa

__all__ = tuple(x for x in dir() if x.endswith("Agent"))



# -*- coding: utf-8 -*-

# Copyright (C) 2007 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Data store and basic position manipulation of a single subtitle."""

import aeidon
import copy

__all__ = ("Subtitle",)


class Subtitle:

    """
    Data store and basic position manipulation of a single subtitle.

    :ivar start: Start position in native units
    :ivar start_time: Start time as string
    :ivar start_frame: Start frame as integer
    :ivar start_seconds: Start seconds as float
    :ivar end: End position in native units
    :ivar end_time: End time as string
    :ivar end_frame: End frame as integer
    :ivar end_seconds: End seconds as float
    :ivar duration: Duration in native units
    :ivar duration_time: Duration in time as string
    :ivar duration_frame: Duration in frames as integer
    :ivar duration_seconds: Duration in seconds as float
    :ivar main_text: Main text
    :ivar tran_text: Translation text
    :ivar calc: :class:`aeidon.Calculator` instance used
    :ivar framerate: :attr:`aeidon.framerates` item
    :ivar mode: :attr:`aeidon.modes` item

    Positions can be set as times, frames or seconds.
    Use :func:`aeidon.as_time`, :func:`aeidon.as_frame` or
    :func:`aeidon.as_seconds` if necessary to ensure correct type.

    Additional format-specific attributes are kept under separate containers,
    e.g. ``ssa`` for Sub Station Alpha formats, accessed as ``subtitle.ssa.*``.
    These containers are lazily created upon first use in order to avoid slow
    instantiation and excessive memory use when handling simpler formats.
    """

    def __init__(self, mode=None, framerate=None):
        """Initialize a :class:`Subtitle` instance."""
        self._start = None
        self._end = None
        self._main_text = ""
        self._tran_text = ""
        self._mode = mode or aeidon.modes.TIME
        self._framerate = framerate or aeidon.framerates.FPS_23_976
        self.calc = aeidon.Calculator(self._framerate)
        self.start = "00:00:00.000"
        self.end = "00:00:00.000"

    def __eq__(self, other):
        """Compare subtitle equality by value."""
        if not isinstance(other, Subtitle):
            raise NotImplementedError
        return (self.start == other.start and
                self.end == other.end and
                self.main_text == other.main_text and
                self.tran_text == other.tran_text and
                self.framerate == other.framerate and
                self.mode == other.mode)

    def __getattr__(self, name):
        """Return lazily instantiated format-specific attribute container."""
        if name in (x.container for x in aeidon.formats):
            # Lazily instantiate a new container.
            container = aeidon.containers.new(name)
            object.__setattr__(self, name, container)
            return container
        raise AttributeError("Invalid container name: {!r}"
                             .format(name))

    def __ge__(self, other):
        """Compare start positions."""
        if self._mode == aeidon.modes.TIME:
            return self.start_seconds >= other.start_seconds
        if self._mode == aeidon.modes.FRAME:
            return self.start_frame >= other.start_frame
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    def __gt__(self, other):
        """Compare start positions."""
        if self._mode == aeidon.modes.TIME:
            return self.start_seconds > other.start_seconds
        if self._mode == aeidon.modes.FRAME:
            return self.start_frame > other.start_frame
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    def __le__(self, other):
        """Compare start positions."""
        if self._mode == aeidon.modes.TIME:
            return self.start_seconds <= other.start_seconds
        if self._mode == aeidon.modes.FRAME:
            return self.start_frame <= other.start_frame
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    def __lt__(self, other):
        """Compare start positions."""
        if self._mode == aeidon.modes.TIME:
            return self.start_seconds < other.start_seconds
        if self._mode == aeidon.modes.FRAME:
            return self.start_frame < other.start_frame
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    def convert_framerate(self, framerate):
        """Set framerate and convert positions to it."""
        coefficient = framerate.value / self._framerate.value
        if self._mode == aeidon.modes.TIME:
            self.start_seconds = self.start_seconds / coefficient
            self.end_seconds = self.end_seconds / coefficient
        if self._mode == aeidon.modes.FRAME:
            self.start_frame = round(coefficient * self.start_frame)
            self.end_frame = round(coefficient * self.end_frame)
        self.framerate = framerate

    def _convert_position(self, value):
        """Return `value` of position in correct mode."""
        if aeidon.is_time(value):
            if self._mode == aeidon.modes.TIME:
                return value
            if self._mode == aeidon.modes.FRAME:
                return self.calc.time_to_frame(value)
        if aeidon.is_frame(value):
            if self._mode == aeidon.modes.TIME:
                return self.calc.frame_to_time(value)
            if self._mode == aeidon.modes.FRAME:
                return value
        if aeidon.is_seconds(value):
            if self._mode == aeidon.modes.TIME:
                return self.calc.seconds_to_time(value)
            if self._mode == aeidon.modes.FRAME:
                return self.calc.seconds_to_frame(value)
        raise ValueError("Invalid type for value: {!r}"
                         .format(type(value)))

    def copy(self):
        """Return a new subtitle instance with the same values."""
        subtitle = Subtitle(self._mode, self._framerate)
        subtitle._start = self._start
        subtitle._end = self._end
        subtitle._main_text = self._main_text
        subtitle._tran_text = self._tran_text
        # Copy all containers that have been instantiated.
        containers = (x.container for x in aeidon.formats)
        for name in set(dir(self)) & set(containers):
            container = copy.deepcopy(getattr(self, name))
            setattr(subtitle, name, container)
        return subtitle

    @property
    def duration(self):
        """Return duration in correct mode."""
        if self._mode == aeidon.modes.TIME:
            return self.duration_time
        if self._mode == aeidon.modes.FRAME:
            return self.duration_frame
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    @duration.setter
    def duration(self, value):
        """Set duration from `value`."""
        value = self._convert_position(value)
        self._end = self.calc.add(self._start, value)

    @property
    def duration_frame(self):
        """Return duration as frames."""
        return self.end_frame - self.start_frame

    @duration_frame.setter
    def duration_frame(self, value):
        """Set duration from `value`."""
        self.duration = aeidon.as_frame(value)

    @property
    def duration_seconds(self):
        """Return duration as seconds."""
        return self.end_seconds - self.start_seconds

    @duration_seconds.setter
    def duration_seconds(self, value):
        """Set duration from `value`."""
        self.duration = aeidon.as_seconds(value)

    @property
    def duration_time(self):
        """Return duration as time."""
        return self.calc.seconds_to_time(self.duration_seconds)

    @duration_time.setter
    def duration_time(self, value):
        """Set duration from `value`."""
        self.duration = aeidon.as_time(value)

    @property
    def end(self):
        """Return end position in correct mode."""
        return self._end

    @end.setter
    def end(self, value):
        """Set end position from `value`."""
        self._end = self._convert_position(value)

    @property
    def end_frame(self):
        """Return end position as frames."""
        if self._mode == aeidon.modes.TIME:
            return self.calc.time_to_frame(self._end)
        if self._mode == aeidon.modes.FRAME:
            return self._end
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    @end_frame.setter
    def end_frame(self, value):
        """Set end position from `value`."""
        self.end = aeidon.as_frame(value)

    @property
    def end_seconds(self):
        """Return end position as seconds."""
        return self.calc.time_to_seconds(self.end_time)

    @end_seconds.setter
    def end_seconds(self, value):
        """Set end position from `value`."""
        self.end = aeidon.as_seconds(value)

    @property
    def end_time(self):
        """Return end position as time."""
        if self._mode == aeidon.modes.TIME:
            return self._end
        if self._mode == aeidon.modes.FRAME:
            return self.calc.frame_to_time(self._end)
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    @end_time.setter
    def end_time(self, value):
        """Set end position from `value`."""
        self.end = aeidon.as_time(value)

    @property
    def framerate(self):
        """Return framerate."""
        return self._framerate

    @framerate.setter
    def framerate(self, value):
        """Set framerate from `value`."""
        self._framerate = value
        self.calc = aeidon.Calculator(value)

    def get_duration(self, mode):
        """Return duration in `mode`."""
        if mode == aeidon.modes.TIME:
            return self.duration_time
        if mode == aeidon.modes.FRAME:
            return self.duration_frame
        raise ValueError("Invalid mode: {!r}"
                         .format(mode))

    def get_end(self, mode):
        """Return end position in `mode`."""
        if mode == aeidon.modes.TIME:
            return self.end_time
        if mode == aeidon.modes.FRAME:
            return self.end_frame
        raise ValueError("Invalid mode: {!r}"
                         .format(mode))

    def get_start(self, mode):
        """Return start position in `mode`."""
        if mode == aeidon.modes.TIME:
            return self.start_time
        if mode == aeidon.modes.FRAME:
            return self.start_frame
        raise ValueError("Invalid mode: {!r}"
                         .format(mode))

    def get_text(self, doc):
        """Return text corresponding to `doc`."""
        if doc == aeidon.documents.MAIN:
            return self._main_text
        if doc == aeidon.documents.TRAN:
            return self._tran_text
        raise ValueError("Invalid document: {!r}"
                         .format(doc))

    def has_container(self, name):
        """Return ``True`` if container has been instantiated."""
        return name in dir(self)

    @property
    def main_text(self):
        """Return main text."""
        return self._main_text

    @main_text.setter
    def main_text(self, value):
        """Set main text from `value`."""
        self._main_text = value

    @property
    def mode(self):
        """Return current position mode."""
        return self._mode

    @mode.setter
    def mode(self, mode):
        """Set current position mode."""
        if mode == aeidon.modes.TIME:
            self._start = self.start_time
            self._end = self.end_time
        if mode == aeidon.modes.FRAME:
            self._start = self.start_frame
            self._end = self.end_frame
        self._mode = mode

    def scale_positions(self, value):
        """Multiply start and end positions by `value`."""
        if self._mode == aeidon.modes.TIME:
            self.start_seconds = self.start_seconds * value
            self.end_seconds = self.end_seconds * value
        if self._mode == aeidon.modes.FRAME:
            self.start_frame = round(self._start * value)
            self.end_frame = round(self._end * value)

    def set_text(self, doc, value):
        """Set text corresponding to `doc` to `value`."""
        if doc == aeidon.documents.MAIN:
            self.main_text = value
        if doc == aeidon.documents.TRAN:
            self.tran_text = value

    def shift_positions(self, value):
        """Add `value` to start and end positions."""
        self._start = self.calc.add(self._start, value)
        self._end = self.calc.add(self._end, value)

    @property
    def start(self):
        """Return start position in correct mode."""
        return self._start

    @start.setter
    def start(self, value):
        """Set start position from `value`."""
        self._start = self._convert_position(value)

    @property
    def start_frame(self):
        """Return start position as frames."""
        if self._mode == aeidon.modes.TIME:
            return self.calc.time_to_frame(self._start)
        if self._mode == aeidon.modes.FRAME:
            return self._start
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    @start_frame.setter
    def start_frame(self, value):
        """Set start position from `value`."""
        self.start = aeidon.as_frame(value)

    @property
    def start_seconds(self):
        """Return start position as seconds."""
        return self.calc.time_to_seconds(self.start_time)

    @start_seconds.setter
    def start_seconds(self, value):
        """Set start position from `value`."""
        self.start = aeidon.as_seconds(value)

    @property
    def start_time(self):
        """Return start position as time."""
        if self._mode == aeidon.modes.TIME:
            return self._start
        if self._mode == aeidon.modes.FRAME:
            return self.calc.frame_to_time(self._start)
        raise ValueError("Invalid mode: {!r}"
                         .format(self._mode))

    @start_time.setter
    def start_time(self, value):
        """Set start position from `value`."""
        self.start = aeidon.as_time(value)

    @property
    def tran_text(self):
        """Return translation text."""
        return self._tran_text

    @tran_text.setter
    def tran_text(self, value):
        """Set translation text from `value`."""
        self._tran_text = value



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Manipulating times and frames."""

import aeidon

from aeidon.i18n import _


class PositionAgent(aeidon.Delegate):

    """Manipulating times and frames."""

    @aeidon.deco.export
    @aeidon.deco.revertable
    def adjust_durations(self, indices=None, speed=None, lengthen=False,
                         shorten=False, minimum=None, maximum=None, gap=None,
                         register=-1):
        """
        Lengthen or shorten durations by changing end positions.

        `indices` can be ``None`` to process all subtitles. `speed` is reading
        speed in characters per second. `lengthen` is ``True`` to increase
        durations to match reading speed. `shorten` is ``True`` to decrease
        durations to match reading speed. `maximum` is the longest allowed
        duration in seconds. `minimum` is the shortest allowed duration in
        seconds. `gap` is seconds to be left between consecutive subtitles.
        Using a gap of at least zero is always a good idea if overlapping
        is not desired. Return changed indices.
        """
        new_indices = []
        new_subtitles = []
        for index in indices or self.get_all_indices():
            start = self.subtitles[index].start_seconds
            end = self.subtitles[index].end_seconds
            if speed is not None:
                length = self.get_text_length(index, aeidon.documents.MAIN)
                optimal_duration = length / speed
                dol = lengthen and end - start < optimal_duration
                dos = shorten  and end - start > optimal_duration
                end = start + optimal_duration if dol or dos else end
            domin = minimum and end - start < minimum
            domax = maximum and end - start > maximum
            end = start + minimum if domin else end
            end = start + maximum if domax else end
            end_max = (self.subtitles[index + 1].start_seconds
                       if index < len(self.subtitles) - 1
                       else 360000)

            dogap = gap is not None and end_max - end < gap
            end = max(start, end_max - gap) if dogap else end
            if end != self.subtitles[index].end_seconds:
                new_indices.append(index)
                subtitle = self.subtitles[index].copy()
                subtitle.end_seconds = end
                new_subtitles.append(subtitle)
        if not new_indices: return []
        self.replace_positions(new_indices, new_subtitles, register=register)
        self.set_action_description(register, _("Adjusting durations"))
        return new_indices

    @aeidon.deco.export
    @aeidon.deco.revertable
    def convert_framerate(self, indices, framerate_in, framerate_out,
                          register=-1):
        """
        Set the value of framerate and convert subtitles to it.

        `indices` can be ``None`` to process all subtitles. `framerate_in` and
        `framerate_out` should be constants from :attr:`aeidon.framerates`.
        """
        new_subtitles = []
        indices = indices or self.get_all_indices()
        self.set_framerate(framerate_in, register=None)
        for index in indices:
            subtitle = self.subtitles[index].copy()
            subtitle.convert_framerate(framerate_out)
            new_subtitles.append(subtitle)
        self.set_framerate(framerate_out)
        self.replace_positions(indices, new_subtitles, register=register)
        self.group_actions(register, 2, _("Converting framerate"))

    def _get_frame_transform(self, p1, p2):
        """Return a formula for linear correction of positions."""
        # Think of this as a linear transformation where input positions
        # are located on the x-axis and output positions on the y-axis.
        x1 = self.subtitles[p1[0]].start_frame
        x2 = self.subtitles[p2[0]].start_frame
        y1 = p1[1]
        y2 = p2[1]
        coefficient = (y2 - y1) / (x2 - x1)
        constant = int(round(-coefficient * x1 + y1, 0))
        return coefficient, constant

    def _get_seconds_transform(self, p1, p2):
        """Return a formula for linear correction of positions."""
        # Think of this as a linear transformation where input positions
        # are located on the x-axis and output positions on the y-axis.
        x1 = self.subtitles[p1[0]].start_seconds
        x2 = self.subtitles[p2[0]].start_seconds
        y1 = p1[1]
        y2 = p2[1]
        coefficient = (y2 - y1) / (x2 - x1)
        constant = -coefficient * x1 + y1
        return coefficient, constant

    def _get_time_transform(self, p1, p2):
        """Return a formula for linear correction of positions."""
        p1 = [p1[0], self.calc.time_to_seconds(p1[1])]
        p2 = [p2[0], self.calc.time_to_seconds(p2[1])]
        return self._get_seconds_transform(p1, p2)

    def _get_transform(self, p1, p2):
        """Return a formula for linear correction of positions."""
        if aeidon.is_time(p1[1]): return self._get_time_transform(p1, p2)
        if aeidon.is_frame(p1[1]): return self._get_frame_transform(p1, p2)
        if aeidon.is_seconds(p1[1]): return self._get_seconds_transform(p1, p2)
        raise ValueError("Bad position argument: {!r}".format(p1))

    @aeidon.deco.export
    @aeidon.deco.revertable
    def set_framerate(self, framerate, register=-1):
        """Set the value of framerate."""
        orig_framerate = self.framerate
        self.framerate = framerate
        self.calc = aeidon.Calculator(framerate)
        for subtitle in self.subtitles:
            subtitle.framerate = framerate
        action = aeidon.RevertableAction(register=register)
        action.docs = tuple(aeidon.documents)
        action.description = _("Setting framerate")
        action.revert_function = self.set_framerate
        action.revert_args = (orig_framerate,)
        self.register_action(action)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def shift_positions(self, indices, value, register=-1):
        """
        Make subtitles appear earlier or later.

        `indices` can be ``None`` to process all subtitles.
        `value` can be any valid position type, negative to make subtitles
        appear ealier, positive to make subtitles appear later.
        """
        new_subtitles = []
        indices = indices or self.get_all_indices()
        for index in indices:
            subtitle = self.subtitles[index].copy()
            subtitle.shift_positions(value)
            new_subtitles.append(subtitle)
        self.replace_positions(indices, new_subtitles, register=register)
        self.set_action_description(register, _("Shifting positions"))

    @aeidon.deco.export
    @aeidon.deco.revertable
    def transform_positions(self, indices, p1, p2, register=-1):
        """
        Change positions by a linear two-point correction.

        `indices` can be ``None`` to process all subtitles.
        `p1` and `p2` should be tuples of index, position.
        """
        new_subtitles = []
        indices = indices or self.get_all_indices()
        coefficient, constant = self._get_transform(p1, p2)
        for index in indices:
            subtitle = self.subtitles[index].copy()
            subtitle.scale_positions(coefficient)
            subtitle.shift_positions(constant)
            new_subtitles.append(subtitle)
        self.replace_positions(indices, new_subtitles, register=register)
        self.set_action_description(register, _("Transforming positions"))



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for editing preferences."""

import aeidon
import gaupol

from aeidon.i18n   import _
from gi.repository import Gtk
from gi.repository import Pango

__all__ = ("PreferencesDialog",)


class EditorPage(aeidon.Delegate, gaupol.BuilderDialog):

    """Editor preferences page."""

    _widgets = [
        "editor_default_font_check",
        "editor_font_button",
        "editor_font_hbox",
        "editor_length_cell_check",
        "editor_length_combo",
        "editor_length_edit_check",
        "editor_length_hbox",
        "editor_spell_check_check",
    ]

    def __init__(self, master, application):
        """Initialize an :class:`EditorPage` instance."""
        aeidon.Delegate.__init__(self, master)
        self._set_attributes(self._widgets, "editor_")
        self.application = application
        self._init_length_combo()
        self._init_values()

    def _get_custom_font(self):
        """Return custom font as string."""
        context = Gtk.Label().get_pango_context()
        font_desc = context.get_font_description()
        font = gaupol.conf.editor.custom_font
        custom_font_desc = Pango.FontDescription(font)
        font_desc.merge(custom_font_desc, True)
        return font_desc.to_string()

    def _init_length_combo(self):
        """Initialize the line length combo box."""
        store = Gtk.ListStore(str)
        for label in (x.label for x in gaupol.length_units):
            store.append((label,))
        self._length_combo.set_model(store)
        renderer = Gtk.CellRendererText()
        self._length_combo.pack_start(renderer, expand=True)
        self._length_combo.add_attribute(renderer, "text", 0)

    def _init_values(self):
        """Initialize default values for widgets."""
        use_custom = gaupol.conf.editor.use_custom_font
        self._default_font_check.set_active(not use_custom)
        self._font_hbox.set_sensitive(use_custom)
        self._font_button.set_font(self._get_custom_font())
        show_cell = gaupol.conf.editor.show_lengths_cell
        show_edit = gaupol.conf.editor.show_lengths_edit
        self._length_hbox.set_sensitive(show_cell or show_edit)
        self._length_cell_check.set_active(show_cell)
        self._length_edit_check.set_active(show_edit)
        self._length_combo.set_active(gaupol.conf.editor.length_unit)
        if gaupol.SpellChecker.available():
            inline = gaupol.conf.spell_check.inline
            self._spell_check_check.set_active(inline)
            self._spell_check_check.set_sensitive(True)
        else: # not available
            self._spell_check_check.set_active(False)
            self._spell_check_check.set_sensitive(False)

    def _on_default_font_check_toggled(self, check_button):
        """Save default font usage."""
        use_custom = not check_button.get_active()
        gaupol.conf.editor.use_custom_font = use_custom
        self._font_hbox.set_sensitive(use_custom)

    def _on_font_button_font_set(self, font_button):
        """Save custom font."""
        gaupol.conf.editor.custom_font = font_button.get_font_name()

    def _on_length_cell_check_toggled(self, check_button):
        """Save line length display on cells."""
        gaupol.conf.editor.show_lengths_cell = check_button.get_active()
        show_cell = gaupol.conf.editor.show_lengths_cell
        show_edit = gaupol.conf.editor.show_lengths_edit
        self._length_hbox.set_sensitive(show_cell or show_edit)

    def _on_length_combo_changed(self, combo_box):
        """Save line length unit."""
        index = combo_box.get_active()
        gaupol.conf.editor.length_unit = gaupol.length_units[index]

    def _on_length_edit_check_toggled(self, check_button):
        """Save line length display on text views."""
        gaupol.conf.editor.show_lengths_edit = check_button.get_active()
        show_cell = gaupol.conf.editor.show_lengths_cell
        show_edit = gaupol.conf.editor.show_lengths_edit
        self._length_hbox.set_sensitive(show_cell or show_edit)

    def _on_spell_check_check_toggled(self, check_button):
        """Save inline spell-check use on text views."""
        gaupol.conf.spell_check.inline = check_button.get_active()


class ExtensionPage(aeidon.Delegate, gaupol.BuilderDialog):

    """Extension activation and preferences page."""

    _widgets = [
        "extensions_about_button",
        "extensions_help_button",
        "extensions_preferences_button",
        "extensions_toolbar",
        "extensions_tree_view",
    ]

    def __init__(self, master, application):
        """Initialize an :class:`ExtensionPage` instance."""
        aeidon.Delegate.__init__(self, master)
        self._set_attributes(self._widgets, "extensions_")
        self.application = application
        self.manager = self.application.extension_manager
        self._init_tree_view()
        self._init_toolbar()
        self._init_values()
        self._init_sensitivities()

    def _get_selected_module(self):
        """Return the selected module in the tree view or ``None``."""
        selection = self._tree_view.get_selection()
        store, itr = selection.get_selected()
        if itr is None: return None
        return store.get_value(itr, 0)

    def _init_sensitivities(self):
        """Initialize button sensitivities."""
        self._about_button.set_sensitive(False)
        self._help_button.set_sensitive(False)
        self._preferences_button.set_sensitive(False)

    def _init_toolbar(self):
        """Initialize the tree view inline toolbar."""
        self._toolbar.set_icon_size(Gtk.IconSize.MENU)
        style = self._toolbar.get_style_context()
        style.add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)

    def _init_tree_view(self):
        """Initialize the tree view."""
        store = Gtk.ListStore(str, bool, str)
        self._tree_view.set_model(store)
        selection = self._tree_view.get_selection()
        selection.set_mode(Gtk.SelectionMode.SINGLE)
        selection.connect("changed", self._on_tree_view_selection_changed)
        renderer = Gtk.CellRendererToggle()
        renderer.props.activatable = True
        renderer.props.xpad = 6
        renderer.connect("toggled", self._on_tree_view_cell_toggled)
        column = Gtk.TreeViewColumn("", renderer, active=1)
        self._tree_view.append_column(column)
        renderer = Gtk.CellRendererText()
        renderer.props.ellipsize = Pango.EllipsizeMode.END
        column = Gtk.TreeViewColumn("", renderer, markup=2)
        self._tree_view.append_column(column)

    def _init_values(self):
        """Initialize default values for widgets."""
        store = self._tree_view.get_model()
        extensions = []
        for module in self.manager.get_modules():
            metadata = self.manager.get_metadata(module)
            if metadata.get_field_boolean("Hidden", False): continue
            markup = "<b>{}</b>\n{}".format(
                metadata.get_name(), metadata.get_description())
            extensions.append((module, markup))
        extensions.sort(key=lambda x: x[1])
        for module, markup in extensions:
            active = module in gaupol.conf.extensions.active
            store.append((module, active, markup))

    def _on_about_button_clicked(self, *args):
        """Construct and show a :class:`Gtk.AboutDialog` for extension."""
        module = self._get_selected_module()
        metadata = self.manager.get_metadata(module)
        dialog = Gtk.AboutDialog()
        dialog.set_transient_for(self._dialog)
        dialog.set_program_name(metadata.get_name())
        dialog.set_comments(metadata.get_description())
        dialog.set_logo_icon_name("io.otsaloma.gaupol")
        if metadata.has_field("Version"):
            dialog.set_version(metadata.get_field("Version"))
        if metadata.has_field("Copyright"):
            copyright = "\n".join(metadata.get_field_list("Copyright"))
            dialog.set_copyright(copyright)
        if metadata.has_field("Website"):
            dialog.set_website(metadata.get_field("Website"))
            label = _("{} Extension Website").format(metadata.get_name())
            dialog.set_website_label(label)
        if metadata.has_field("Authors"):
            dialog.set_authors(metadata.get_field_list("Authors"))
        gaupol.util.flash_dialog(dialog)

    def _on_help_button_clicked(self, *args):
        """Show extension's own documentation."""
        module = self._get_selected_module()
        self.manager.show_help(module)

    def _on_preferences_button_clicked(self, *args):
        """Show extension's preferences dialog."""
        module = self._get_selected_module()
        self.manager.show_preferences_dialog(module, self._dialog)

    def _on_tree_view_cell_toggled(self, renderer, path):
        """Activate or deactivate toggled extension."""
        store = self._tree_view.get_model()
        module = store[path][0]
        active = store[path][1]
        if active:
            self.manager.teardown_extension(module)
            gaupol.conf.extensions.active.remove(module)
        else:
            self.manager.setup_extension(module)
            gaupol.conf.extensions.active.append(module)
        for row in store:
            row[1] = self.manager.is_active(row[0])
        selection = self._tree_view.get_selection()
        selection.emit("changed")

    def _on_tree_view_selection_changed(self, *args):
        """Set sensitivities of buttons."""
        module = self._get_selected_module()
        have_selection = self._get_selected_module() is not None
        self._about_button.set_sensitive(have_selection)
        if module in gaupol.conf.extensions.active:
            has_help = self.manager.has_help(module)
            self._help_button.set_sensitive(has_help)
            has_preferences = self.manager.has_preferences_dialog(module)
            self._preferences_button.set_sensitive(has_preferences)
        else: # Cannot query unimported extensions.
            self._help_button.set_sensitive(False)
            self._preferences_button.set_sensitive(False)


class FilePage(aeidon.Delegate, gaupol.BuilderDialog):

    """File preferences page."""

    _widgets = [
        "file_add_button",
        "file_auto_check",
        "file_down_button",
        "file_locale_check",
        "file_remove_button",
        "file_toolbar",
        "file_tree_view",
        "file_up_button",
    ]

    def __init__(self, master, application):
        """Initialize a :class:`FilePage` instance."""
        aeidon.Delegate.__init__(self, master)
        self._set_attributes(self._widgets, "file_")
        self.application = application
        self._init_tree_view()
        self._init_toolbar()
        self._init_values()

    def _get_selected_row(self):
        """Return the selected row in the tree view or ``None``."""
        selection = self._tree_view.get_selection()
        store, itr = selection.get_selected()
        if itr is None: return None
        path = store.get_path(itr)
        return gaupol.util.tree_path_to_row(path)

    def _init_toolbar(self):
        """Initialize the tree view inline toolbar."""
        self._toolbar.set_icon_size(Gtk.IconSize.MENU)
        style = self._toolbar.get_style_context()
        style.add_class(Gtk.STYLE_CLASS_INLINE_TOOLBAR)
        theme = Gtk.IconTheme.get_default()
        # Tool buttons in the UI file are specified as symbolic icons
        # by name, found in adwaita-icon-theme. If missing in another
        # theme fall back to non-symbolic icons.
        if not all((theme.has_icon(self._add_button.get_icon_name()),
                    theme.has_icon(self._remove_button.get_icon_name()),
                    theme.has_icon(self._up_button.get_icon_name()),
                    theme.has_icon(self._down_button.get_icon_name()))):

            self._add_button.set_icon_name("list-add")
            self._remove_button.set_icon_name("list-remove")
            self._up_button.set_icon_name("go-up")
            self._down_button.set_icon_name("go-down")

    def _init_tree_view(self):
        """Initialize the fallback encoding tree view."""
        selection = self._tree_view.get_selection()
        selection.set_mode(Gtk.SelectionMode.SINGLE)
        update = lambda x, self: self._set_sensitivities()
        selection.connect("changed", update, self)
        store = Gtk.ListStore(str)
        self._tree_view.set_model(store)
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("", renderer, text=0)
        self._tree_view.append_column(column)

    def _init_values(self):
        """Initialize default values for widgets."""
        self._auto_check.set_active(gaupol.conf.encoding.try_auto)
        self._auto_check.set_sensitive(aeidon.util.chardet_available())
        self._locale_check.set_active(gaupol.conf.encoding.try_locale)
        self._reload_tree_view()

    def _on_add_button_clicked(self, *args):
        """Add a new fallback encoding."""
        dialog = gaupol.EncodingDialog(self._dialog)
        response = gaupol.util.run_dialog(dialog)
        encoding = dialog.get_encoding()
        dialog.destroy()
        if response != Gtk.ResponseType.OK: return
        if encoding is None: return
        if encoding in gaupol.conf.encoding.fallback: return
        gaupol.conf.encoding.fallback.append(encoding)
        self._reload_tree_view()
        self._tree_view.grab_focus()
        store = self._tree_view.get_model()
        self._tree_view.set_cursor(len(store) - 1)

    def _on_auto_check_toggled(self, check_button):
        """Save encoding auto-detection usage."""
        gaupol.conf.encoding.try_auto = check_button.get_active()

    def _on_down_button_clicked(self, *args):
        """Move the selected fallback encoding down."""
        row = self._get_selected_row()
        encodings = gaupol.conf.encoding.fallback
        encodings.insert(row + 1, encodings.pop(row))
        self._reload_tree_view()
        self._tree_view.grab_focus()
        self._tree_view.set_cursor(row + 1)

    def _on_locale_check_toggled(self, check_button):
        """Save locale encoding usage."""
        gaupol.conf.encoding.try_locale = check_button.get_active()

    def _on_remove_button_clicked(self, *args):
        """Remove the selected fallback encoding."""
        row = self._get_selected_row()
        gaupol.conf.encoding.fallback.pop(row)
        self._reload_tree_view()
        self._tree_view.grab_focus()
        store = self._tree_view.get_model()
        if len(store) <= 0: return
        self._tree_view.set_cursor(max(row-1, 0))

    def _on_up_button_clicked(self, *args):
        """Move the selected fallback encoding up."""
        row = self._get_selected_row()
        encodings = gaupol.conf.encoding.fallback
        encodings.insert(row-1, encodings.pop(row))
        self._reload_tree_view()
        self._tree_view.grab_focus()
        self._tree_view.set_cursor(row-1)

    def _reload_tree_view(self):
        """Reload the fallback encoding tree view."""
        store = self._tree_view.get_model()
        store.clear()
        for encoding in gaupol.conf.encoding.fallback:
            store.append((aeidon.encodings.code_to_long_name(encoding),))
        self._set_sensitivities()

    def _set_sensitivities(self):
        """Set the fallback tree view button sensitivities."""
        store = self._tree_view.get_model()
        row = self._get_selected_row()
        row = -1 if row is None else row
        self._remove_button.set_sensitive(row >= 0)
        self._up_button.set_sensitive(row > 0)
        self._down_button.set_sensitive(0 <= row < len(store) - 1)


class PreviewPage(aeidon.Delegate, gaupol.BuilderDialog):

    """Preview preferences page."""

    _widgets = [
        "preview_app_combo",
        "preview_command_entry",
        "preview_force_utf_8_check",
        "preview_offset_spin",
    ]

    def __init__(self, master, application):
        """Initialize a :class:`PreviewPage` instance."""
        aeidon.Delegate.__init__(self, master)
        self._set_attributes(self._widgets, "preview_")
        self.application = application
        self._init_app_combo()
        self._init_values()

    def _init_app_combo(self):
        """Initialize the application combo box."""
        store = Gtk.ListStore(str)
        self._app_combo.set_model(store)
        for label in (x.label for x in aeidon.players):
            store.append((label,))
        store.append((gaupol.COMBO_SEPARATOR,))
        store.append((_("Custom"),))
        renderer = Gtk.CellRendererText()
        self._app_combo.pack_start(renderer, expand=True)
        self._app_combo.add_attribute(renderer, "text", 0)
        func = gaupol.util.separate_combo
        self._app_combo.set_row_separator_func(func, None)

    def _init_values(self):
        """Initialize default values for widgets."""
        store = self._app_combo.get_model()
        self._app_combo.set_active((len(store) - 1 if
                                    gaupol.conf.preview.use_custom_command else
                                    gaupol.conf.preview.player))

        self._command_entry.set_text(gaupol.util.get_preview_command())
        editable = gaupol.conf.preview.use_custom_command
        self._command_entry.set_editable(editable)
        self._force_utf_8_check.set_active(gaupol.conf.preview.force_utf_8)
        self._offset_spin.set_value(gaupol.conf.preview.offset)

    def _on_app_combo_changed(self, combo_box):
        """Save video player and show it's command."""
        index = combo_box.get_active()
        use_custom = not index in aeidon.players
        gaupol.conf.preview.use_custom_command = use_custom
        if not use_custom:
            gaupol.conf.preview.player = aeidon.players[index]
        self._command_entry.set_text(gaupol.util.get_preview_command())
        editable = gaupol.conf.preview.use_custom_command
        self._command_entry.set_editable(editable)

    def _on_command_entry_changed(self, entry):
        """Save custom command."""
        if not gaupol.conf.preview.use_custom_command: return
        gaupol.conf.preview.custom_command = entry.get_text()

    def _on_force_utf_8_check_toggled(self, check_button):
        """Save forced UTF-8 preview setting."""
        gaupol.conf.preview.force_utf_8 = check_button.get_active()
        self._command_entry.set_text(gaupol.util.get_preview_command())

    def _on_offset_spin_value_changed(self, spin_button):
        """Save start position offset."""
        gaupol.conf.preview.offset = spin_button.get_value()


class VideoPage(aeidon.Delegate, gaupol.BuilderDialog):

    """Video preferences page."""

    _widgets = [
        "video_seek_spin",
        "video_subtitle_bg_check",
        "video_subtitle_color_button",
        "video_subtitle_font_button",
        "video_time_bg_check",
        "video_time_color_button",
        "video_time_font_button",
    ]

    def __init__(self, master, application):
        """Initialize a :class:`PreviewPage` instance."""
        aeidon.Delegate.__init__(self, master)
        self._set_attributes(self._widgets, "video_")
        self.application = application
        self.conf = gaupol.conf.video_player
        self._init_values()

    def _get_custom_font(self, font):
        """Return custom font as string."""
        context = Gtk.Label().get_pango_context()
        font_desc = context.get_font_description()
        custom_font_desc = Pango.FontDescription(font)
        font_desc.merge(custom_font_desc, True)
        return font_desc.to_string()

    def _init_values(self):
        """Initialize default values for widgets."""
        font = self._get_custom_font(self.conf.subtitle_font)
        self._subtitle_font_button.set_font(font)
        font = self._get_custom_font(self.conf.time_font)
        self._time_font_button.set_font(font)
        self._subtitle_bg_check.set_active(self.conf.subtitle_background)
        color = gaupol.util.hex_to_rgba(self.conf.subtitle_color)
        color.alpha = self.conf.subtitle_alpha
        self._subtitle_color_button.set_rgba(color)
        self._time_bg_check.set_active(self.conf.time_background)
        color = gaupol.util.hex_to_rgba(self.conf.time_color)
        color.alpha = self.conf.time_alpha
        self._time_color_button.set_rgba(color)
        self._seek_spin.set_value(self.conf.seek_length)

    def _on_seek_spin_value_changed(self, spin_button):
        """Save seek length."""
        self.conf.seek_length = spin_button.get_value()

    def _on_subtitle_bg_check_toggled(self, check_button):
        """Save subtitle background use."""
        self.conf.subtitle_background = check_button.get_active()

    def _on_subtitle_color_button_color_set(self, color_button):
        """Save subtitle color."""
        color = color_button.get_rgba()
        self.conf.subtitle_color = gaupol.util.rgba_to_hex(color)
        self.conf.subtitle_alpha = color.alpha

    def _on_subtitle_font_button_font_set(self, font_button):
        """Save subtitle font."""
        self.conf.subtitle_font = font_button.get_font_name()

    def _on_time_bg_check_toggled(self, check_button):
        """Save time background use."""
        self.conf.time_background = check_button.get_active()

    def _on_time_color_button_color_set(self, color_button):
        """Save time color."""
        color = color_button.get_rgba()
        self.conf.time_color = gaupol.util.rgba_to_hex(color)
        self.conf.time_alpha = color.alpha

    def _on_time_font_button_font_set(self, font_button):
        """Save time font."""
        self.conf.time_font = font_button.get_font_name()


class PreferencesDialog(gaupol.BuilderDialog):

    """Dialog for editing preferences."""

    _widgets = ["notebook"]

    def __init__(self, parent, application):
        """Initialize a :class:`PreferencesDialog` instance."""
        gaupol.BuilderDialog.__init__(self,
                                      "preferences-dialog.ui",
                                      connect_signals=False)

        self._editor_page = EditorPage(self, application)
        self._extension_page = ExtensionPage(self, application)
        self._file_page = FilePage(self, application)
        self._preview_page = PreviewPage(self, application)
        self._video_page = VideoPage(self, application)
        self._builder.connect_signals(self._get_callbacks())
        self._init_dialog(parent)

    def _get_callbacks(self):
        """Return a dictionary mapping names to callback methods."""
        callbacks = {}
        for page in (self._editor_page,
                     self._extension_page,
                     self._file_page,
                     self._preview_page,
                     self._video_page):

            for name in (x for x in dir(page) if x.startswith("_on_")):
                callbacks[name] = getattr(page, name)
        return callbacks

    def _init_dialog(self, parent):
        """Initialize the dialog."""
        self.set_default_response(Gtk.ResponseType.CLOSE)
        self.set_transient_for(parent)
        self.set_modal(True)



# -*- coding: utf-8 -*-

# Copyright (C) 2013 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Audio actions for :class:`gaupol.Application`."""

import aeidon
import gaupol

class SetAudioLanguageAction(gaupol.RadioAction):
    def __new__(cls):
        action = gaupol.RadioAction.new("set-audio-language")
        action.__class__ = cls
        return action
    def __init__(self):
        gaupol.RadioAction.__init__(self, "set-audio-language")
        self.set_state(str(0))
    def _affirm_doable(self, application, page, selected_rows):
        aeidon.util.affirm(application.player is not None)
        aeidon.util.affirm(application.player.ready)

class VolumeDownAction(gaupol.Action):
    def __init__(self):
        gaupol.Action.__init__(self, "volume-down")
        self.accelerators = ["<Control>minus", "<Control>KP_Subtract"]
        self.action_group = "unsafe"
    def _affirm_doable(self, application, page, selected_rows):
        aeidon.util.affirm(application.player is not None)
        aeidon.util.affirm(application.player.ready)

class VolumeUpAction(gaupol.Action):
    def __init__(self):
        gaupol.Action.__init__(self, "volume-up")
        self.accelerators = ["<Control>plus", "<Control>KP_Add"]
        self.action_group = "unsafe"
    def _affirm_doable(self, application, page, selected_rows):
        aeidon.util.affirm(application.player is not None)
        aeidon.util.affirm(application.player.ready)

__all__ = tuple(x for x in dir() if x.endswith("Action"))



# -*- coding: utf-8 -*-

# Copyright (C) 2017 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""LRC file."""

import aeidon
import re

__all__ = ("LRC",)


class LRC(aeidon.SubtitleFile):

    """
    LRC file.

    https://en.wikipedia.org/wiki/LRC_(file_format)
    """

    format = aeidon.formats.LRC
    mode = aeidon.modes.TIME
    _re_line = re.compile(r"^\[(-?\d\d:\d\d.\d\d)\](.*)$")

    def read(self):
        """
        Read file and return subtitles.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        """
        self.header = ""
        subtitles = [self._get_subtitle()]
        for line in self._read_lines():
            match = self._re_line.match(line)
            if match is None and len(subtitles) == 1:
                # Read line into file header.
                if self.header:
                    self.header += "\n"
                self.header += line
            elif match is not None:
                subtitle = self._get_subtitle()
                normalize = subtitle.calc.normalize_time
                subtitle.start_time = normalize(match.group(1))
                subtitles[-1].end_time = subtitle.start_time
                subtitle.main_text = match.group(2) or ""
                subtitles.append(subtitle)
        subtitles[-1].duration_seconds = 5
        return subtitles[1:]

    def write_to_file(self, subtitles, doc, f):
        """
        Write `subtitles` from `doc` to file `f`.

        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        if self.header.strip():
            f.write(self.header.strip() + "\n\n")
        for subtitle in subtitles:
            start = subtitle.calc.round(subtitle.start_time, 2)
            sign = "-" if start.startswith("-") else ""
            first = 4 if start.startswith("-") else 3
            start = sign + start[first:-1]
            text = subtitle.get_text(doc).replace("\n", " ")
            f.write("[{}]{}\n".format(start, text))



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Base class for text markup."""

import aeidon
import re

__all__ = ("Markup",)


class Markup(aeidon.Singleton):

    """
    Base class for text markup.

    Markup conversions between different formats are done via an internal
    format, which has the following BBcode-style tags with angle brackets.
    Conversions are best done via the ``_decode_*`` and ``_encode_*`` methods
    rather than hard-coding internal tags in regular expression substitutions.

     * ``<b>...................</b>``
     * ``<i>...................</i>``
     * ``<u>...................</u>``
     * ``<color=#RRGGBB>...</color>``
     * ``<font=NAME>........</font>``
     * ``<size=POINTS>......</size>``

    :class:`Markup` is effectively equivalent to a format with no markup and
    can therefore be merely subclassed with a ``pass``-statement by formats
    that do not implement any markup. The caller of any tagging methods, e.g.
    :meth:`bolden`, must be prepared to handle :exc:`NotImplementedError`.
    """

    _flags = re.DOTALL | re.MULTILINE
    format = aeidon.formats.NONE

    def bolden(self, text, bounds=None):
        """Return bolded `text`."""
        raise NotImplementedError

    def clean(self, text):
        """
        Return `text` with less ugly markup.

        Subclasses can implement this to, for example, remove redundant markup,
        finetune tag positioning or to join or split tags; in general, whatever
        that changes the style of the markup but not that of the text.
        """
        return text

    def colorize(self, text, color, bounds=None):
        """Return `text` colorized to hexadecimal value."""
        raise NotImplementedError

    def decode(self, text):
        """Return `text` with markup converted from this to internal format."""
        text = self._pre_decode(text)
        text = self._main_decode(text)
        return self._post_decode(text)

    def _decode_apply(self, text, regex, replacement, groups):
        """
        Return `text` with all matches of `regex` replaced.

        `replacement` may contain one or more of ``{}``, which are replaced
        with parts of the match as defined by `groups`, a ``tuple`` of numbers.
        """
        orig_text = text
        match = regex.search(text)
        if match is None: return text
        a, z = match.span()
        new = replacement.format(*tuple(map(match.group, groups)))
        text = "".join((text[:a], new, text[z:]))
        if text == orig_text: return text
        return self._decode_apply(text, regex, replacement, groups)

    def _decode_b(self, text, pattern, target, flags=0):
        """Return `text` with bold markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<b>{}</b>"
        return self._decode_apply(text, regex, replacement, (target,))

    def _decode_c(self, text, pattern, value, target, flags=0):
        """Return `text` with color markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<color=#{}>{}</color>"
        return self._decode_apply(text, regex, replacement, (value, target))

    def _decode_f(self, text, pattern, value, target, flags=0):
        """Return `text` with font markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<font={}>{}</font>"
        return self._decode_apply(text, regex, replacement, (value, target))

    def _decode_i(self, text, pattern, target, flags=0):
        """Return `text` with italic markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<i>{}</i>"
        return self._decode_apply(text, regex, replacement, (target,))

    def _decode_s(self, text, pattern, value, target, flags=0):
        """Return `text` with size markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<size={}>{}</size>"
        return self._decode_apply(text, regex, replacement, (value, target))

    def _decode_u(self, text, pattern, target, flags=0):
        """Return `text` with underline markup converted to internal format."""
        regex = self._get_regex(pattern, flags)
        replacement = "<u>{}</u>"
        return self._decode_apply(text, regex, replacement, (target,))

    def encode(self, text):
        """Return `text` with markup converted from internal to this format."""
        text = self._encode_b(text)
        text = self._encode_c(text)
        text = self._encode_f(text)
        text = self._encode_i(text)
        text = self._encode_s(text)
        return self._encode_u(text)

    def _encode_apply(self, text, regex, method, target, value=None):
        """
        Return `text` with internal tags removed and `method` applied.

        `method` should be one the tagging methods, e.g. meth:`bolden`.
        `target` and `value` should be group numbers in `regex`.
        """
        orig_text = text
        match = regex.search(text)
        if match is None: return text
        text = regex.sub(r"\{}".format(target), text, 1)
        a = match.start()
        z = a + len(match.group(target))
        args = (text, (a, z))
        if value is not None:
            args = (text, match.group(value), (a, z))
        with aeidon.util.silent(NotImplementedError):
            text = method(*args)
        if text == orig_text: return text
        return self._encode_apply(text, regex, method, target, value)

    def _encode_b(self, text):
        """Return `text` with bold markup converted to this format."""
        regex = self._get_regex(r"<b>(.*?)</b>")
        return self._encode_apply(text, regex, self.bolden, 1)

    def _encode_c(self, text):
        """Return `text` with color markup converted to this format."""
        regex = self._get_regex(r"<color=#([a-fA-F0-9]{6})>(.*?)</color>")
        return self._encode_apply(text, regex, self.colorize, 2, 1)

    def _encode_f(self, text):
        """Return `text` with font markup converted to this format."""
        regex = self._get_regex(r"<font=(.+?)>(.*?)</font>")
        return self._encode_apply(text, regex, self.fontify, 2, 1)

    def _encode_i(self, text):
        """Return `text` with italic markup converted to this format."""
        regex = self._get_regex(r"<i>(.*?)</i>")
        return self._encode_apply(text, regex, self.italicize, 1)

    def _encode_s(self, text):
        """Return `text` with size markup converted to this format."""
        regex = self._get_regex(r"<size=(\d+)>(.*?)</size>")
        return self._encode_apply(text, regex, self.scale, 2, 1)

    def _encode_u(self, text):
        """Return `text` with underline markup converted to this format."""
        regex = self._get_regex(r"<u>(.*?)</u>")
        return self._encode_apply(text, regex, self.underline, 1)

    def fontify(self, text, font, bounds=None):
        """Return `text` changed to `font`."""
        raise NotImplementedError

    @aeidon.deco.memoize(100)
    def _get_regex(self, pattern, flags=0):
        """Return compiled regular expression from cache."""
        flags = self._flags | flags
        return re.compile(pattern, flags)

    @property
    def italic_tag(self):
        """Regular expression for an italic markup tag or ``None``."""
        return None

    def italicize(self, text, bounds=None):
        """Return italicized `text`."""
        raise NotImplementedError

    def _main_decode(self, text):
        """Return `text` with decodable markup decoded."""
        return text

    def _post_decode(self, text):
        """Return `text` with markup finalized after decoding."""
        return text

    def _pre_decode(self, text):
        """Return `text` with markup prepared for decoding."""
        return text

    def scale(self, text, size, bounds=None):
        """Return `text` scaled to `size`."""
        raise NotImplementedError

    def _substitute(self, text, pattern, replacement, flags=0):
        """Return `text` with matches of `pattern` replaced."""
        regex = self._get_regex(pattern, flags)
        return regex.sub(replacement, text)

    @property
    def tag(self):
        """Regular expression for any markup tag or ``None``."""
        return None

    def underline(self, text, bounds=None):
        """Return underlined `text`."""
        raise NotImplementedError



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Saving documents."""

import aeidon
import gaupol
import os

from aeidon.i18n   import _
from gi.repository import Gtk


class SaveAgent(aeidon.Delegate):

    """Saving documents."""

    @aeidon.deco.export
    def _on_save_all_documents_activate(self, *args):
        """Save all open documents."""
        for page in self.pages:
            with aeidon.util.silent(gaupol.Default):
                self.save_main(page)
            if page.project.tran_changed is not None:
                with aeidon.util.silent(gaupol.Default):
                    self.save_translation(page)
        self.update_gui()

    @aeidon.deco.export
    def _on_save_all_documents_as_activate(self, *args):
        """Save all open documents with different properties."""
        modes = []
        for page in self.pages:
            if page.project.main_file is not None:
                modes.append(page.project.main_file.mode)
        dialog = gaupol.MultiSaveDialog(self.window, self, modes)
        gaupol.util.flash_dialog(dialog)
        self.update_gui()

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_save_main_document_activate(self, *args):
        """Save the current main document."""
        page = self.get_current_page()
        self.save_main(page)
        self.update_gui()

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_save_main_document_as_activate(self, *args):
        """Save the current main document with a different name."""
        page = self.get_current_page()
        self.save_main_as(page)
        self.update_gui()

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_save_translation_document_activate(self, *args):
        """Save the current translation document."""
        page = self.get_current_page()
        self.save_translation(page)
        self.update_gui()

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_save_translation_document_as_activate(self, *args):
        """Save the current translation document with a different name."""
        page = self.get_current_page()
        self.save_translation_as(page)
        self.update_gui()

    @aeidon.deco.export
    def save(self, page, doc):
        """Save `doc` of `page` to file."""
        if doc == aeidon.documents.MAIN:
            return self.save_main(page)
        if doc == aeidon.documents.TRAN:
            return self.save_translation(page)
        raise ValueError("Invalid document: {!r}"
                         .format(doc))

    def _save_document(self, page, doc, file=None):
        """Save document to `file` or raise :exc:`gaupol.Default`."""
        try:
            file = file or page.project.get_file(doc)
            gaupol.util.set_cursor_busy(self.window)
            return page.project.save(doc, file)
        except IOError as error:
            gaupol.util.set_cursor_normal(self.window)
            basename = os.path.basename(file.path)
            self._show_io_error_dialog(basename, str(error))
        except UnicodeError:
            gaupol.util.set_cursor_normal(self.window)
            basename = os.path.basename(file.path)
            self._show_encoding_error_dialog(basename, file.encoding)
        finally:
            gaupol.util.set_cursor_normal(self.window)
        raise gaupol.Default

    @aeidon.deco.export
    def save_main(self, page):
        """Save the main document of `page` to file."""
        if (page.project.main_file is None or
            page.project.main_file.path is None or
            page.project.main_file.encoding is None):
            return self.save_main_as(page)
        self._save_document(page, aeidon.documents.MAIN)
        self.emit("page-saved", self, page)

    @aeidon.deco.export
    def save_main_as(self, page, file=None):
        """Save the main document of `page` to a selected file."""
        if file is None:
            file = page.project.main_file
            file = self._select_file(_("Save As"), page, file)
        self._save_document(page, aeidon.documents.MAIN, file)
        self.emit("page-saved", self, page)
        path = page.project.main_file.path
        format = page.project.main_file.format
        self.add_to_recent_files(path, format, aeidon.documents.MAIN)
        self.flash_message(_('Saved main document as "{}"')
                           .format(os.path.basename(path)))

    @aeidon.deco.export
    def save_translation(self, page):
        """Save the translation document of `page` to file."""
        if (page.project.tran_file is None or
            page.project.tran_file.path is None or
            page.project.tran_file.encoding is None):
            return self.save_translation_as(page)
        self._save_document(page, aeidon.documents.TRAN)
        self.emit("page-saved", self, page)

    @aeidon.deco.export
    def save_translation_as(self, page, file=None):
        """Save the translation document of `page` to a selected file."""
        if file is None:
            file = page.project.tran_file
            file = self._select_file(_("Save Translation As"), page, file)
        self._save_document(page, aeidon.documents.TRAN, file)
        self.emit("page-saved", self, page)
        path = page.project.tran_file.path
        format = page.project.tran_file.format
        self.add_to_recent_files(path, format, aeidon.documents.TRAN)
        self.flash_message(_('Saved translation document as "{}"')
                           .format(os.path.basename(path)))

    def _select_file(self, title, page, file=None):
        """Select a file to save or raise :exc:`gaupol.Default`."""
        gaupol.util.set_cursor_busy(self.window)
        mode = page.project.get_mode()
        dialog = gaupol.SaveDialog(self.window, title, mode)
        if file is not None:
            dialog.set_name(file.path)
            dialog.set_format(file.format)
            dialog.set_encoding(file.encoding)
            dialog.set_newline(file.newline)
            dialog.set_framerate(page.project.framerate)
        gaupol.util.set_cursor_normal(self.window)
        response = gaupol.util.run_dialog(dialog)
        format = dialog.get_format()
        path = dialog.get_filename()
        encoding = dialog.get_encoding()
        newline = dialog.get_newline()
        framerate = dialog.get_framerate()
        dialog.destroy()
        if response != Gtk.ResponseType.OK:
            raise gaupol.Default
        if format.mode != mode:
            # Set framerate to the selected one.
            self.set_current_page(page)
            action = self.get_action("set-framerate")
            action.activate(str(framerate))
        gaupol.util.iterate_main()
        return aeidon.files.new(format, path, encoding, newline)

    def _show_encoding_error_dialog(self, basename, codec):
        """Show an error dialog after failing to encode file."""
        codec = aeidon.encodings.code_to_name(codec)
        title = _('Failed to encode file "{basename}" with codec "{codec}"').format(**locals())
        message = _("Please try to save the file with a different character encoding.")
        dialog = gaupol.ErrorDialog(self.window, title, message)
        dialog.add_button(_("_OK"), Gtk.ResponseType.OK)
        dialog.set_default_response(Gtk.ResponseType.OK)
        gaupol.util.flash_dialog(dialog)

    def _show_io_error_dialog(self, basename, message):
        """Show an error dialog after failing to write file."""
        title = _('Failed to save file "{}"').format(basename)
        dialog = gaupol.ErrorDialog(self.window, title, message)
        dialog.add_button(_("_OK"), Gtk.ResponseType.OK)
        dialog.set_default_response(Gtk.ResponseType.OK)
        gaupol.util.flash_dialog(dialog)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Storing text to the clipboard and pasting from it."""

import aeidon

from aeidon.i18n import _


class ClipboardAgent(aeidon.Delegate):

    """Storing text to the clipboard and pasting from it."""

    @aeidon.deco.export
    def copy_texts(self, indices, doc):
        """Copy texts to the clipboard."""
        self.clipboard.clear()
        for index in range(min(indices), max(indices) + 1):
            subtitle = self.subtitles[index]
            text = subtitle.get_text(doc) if index in indices else None
            self.clipboard.append(text)

    @aeidon.deco.export
    @aeidon.deco.revertable
    def cut_texts(self, indices, doc, register=-1):
        """Cut texts to the clipboard."""
        self.copy_texts(indices, doc)
        self.clear_texts(indices, doc, register=register)
        self.set_action_description(register, _("Cutting texts"))

    @aeidon.deco.export
    @aeidon.deco.revertable
    def paste_texts(self, index, doc, register=-1):
        """Paste texts from the clipboard and return pasted indices."""
        texts = self.clipboard.get_texts()
        length = len(self.subtitles)
        new_count = len(texts) - (length - index)
        if new_count > 0:
            indices = list(range(length, length + new_count))
            self.insert_subtitles(indices, register=register)
        indices = [index+i for i in range(len(texts)) if texts[i] is not None]
        new_texts = [x for x in texts if x is not None]
        self.replace_texts(indices, doc, new_texts, register=register)
        if new_count > 0:
            self.group_actions(register, 2, "")
        self.set_action_description(register, _("Pasting texts"))
        return tuple(indices)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for editing text of a single subtitle."""

import aeidon
import gaupol

from aeidon.i18n   import _
from gi.repository import GObject
from gi.repository import Gtk


class TextEditDialog(Gtk.Dialog):

    """Dialog for editing text of a single subtitle."""

    def __init__(self, parent, text=""):
        """Initialize a :class:`TextEditDialog` instance."""
        GObject.GObject.__init__(self, use_header_bar=True)
        self._text_view = Gtk.TextView()
        self._init_dialog(parent)
        self._init_text_view()
        self.set_text(text)

    def get_text(self):
        """Return text in the text view."""
        text_buffer = self._text_view.get_buffer()
        start, end = text_buffer.get_bounds()
        return text_buffer.get_text(start, end, False)

    def _init_dialog(self, parent):
        """Initialize the dialog."""
        self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
        self.add_button(_("_OK"), Gtk.ResponseType.OK)
        self.set_default_response(Gtk.ResponseType.OK)
        self.set_transient_for(parent)
        self.set_modal(True)
        self.set_title(_("Edit Text"))

    def _init_text_view(self):
        """Initialize the text view."""
        gaupol.util.prepare_text_view(self._text_view)
        self._text_view.set_wrap_mode(Gtk.WrapMode.NONE)
        self._text_view.set_accepts_tab(False)
        self._text_view.set_left_margin(6)
        self._text_view.set_right_margin(6)
        with aeidon.util.silent(AttributeError):
            # Top and bottom margins available since GTK 3.18.
            self._text_view.set_top_margin(6)
            self._text_view.set_bottom_margin(6)
        scroller = Gtk.ScrolledWindow()
        scroller.set_policy(*((Gtk.PolicyType.AUTOMATIC,)*2))
        scroller.set_shadow_type(Gtk.ShadowType.NONE)
        scroller.add(self._text_view)
        box = self.get_content_area()
        gaupol.util.pack_start_expand(box, scroller)
        gaupol.util.scale_to_size(self._text_view,
                                  nchar=55,
                                  nlines=7,
                                  font="custom")

        box.show_all()

    def set_text(self, text):
        """Set `text` to the text view."""
        text_buffer = self._text_view.get_buffer()
        text_buffer.set_text(text)
